/*
	SNDXFORM.H	- Functions for sample processing

	(c) 1997 Martin Pitt as Public Domain

	Version: 1.0

   --------

	common parameters:

	Sample src   : source sample
	Sample dest  : buffer for destination sample
	unsigned len : length of source sample
*/

#ifndef SNDXFORM_H
#define SNDXFORM_H

typedef unsigned char* Sample;

//* format conversion ***********************************************

// converts between 8 bit unsigned and signed (overwrites src!)
void sampleToggleSign (Sample src, unsigned len);


//* filters (format: 8 bit unsigned) ********************************

void sampleLoPass (Sample src, Sample dest, unsigned len);
void sampleHiPass (Sample src, Sample dest, unsigned len);

/* frq:		sample rate
   midFrq:      frequency filter is damping least at
   width:	width of passed frequency window

   Attention: this filter does not damp low frequencies very good -> use
   	      low pass instead
*/
void sampleMedPass (Sample src, Sample dest, unsigned len,
                   unsigned frq, unsigned midFrq, unsigned width);



/* Just like sampleMedPass, but with precalculated coefficients -> faster!

                      -2Pi*width/frq                              -Pi*width/frq
  a= 100-b-c; b= -100e               ; c= 200cos(2Pi*midFrq/frq)*e
*/  
void sampleMedPassX (Sample src, Sample dest, unsigned len,
                     int a, int b, int c);


//* frequency analysis **********************************************

/*
  r: real part of input function (gets overwritten by real part of fft)
  i: imaginary part of input function (overwr. by imag. part of fft)
  l: fft class (i.e. length of sample = 2^l)
*/
void fft (float *r, float *i, unsigned char l);

//* convenience (shorthand) functions *
void fft8 (Sample src, Sample dest);
void fft9 (Sample src, Sample dest);

/*
  dest: 	buffer for cepstrum coefficients (numCfc bytes)
  firstCfc:	number of cepstrum coefficient to start with (0 to 2^l - 1)
  numCfc:	number of coefficients to calculate (0 to 2^l - 1 - firstCfc)
  amplify:	amplification factor (1 = no amplification)
*/
void cepstrum (const Sample src, unsigned char l, Sample dest,
	       unsigned firstCfc, unsigned numCfc, float amplify = 1);

#endif
