/*
	sound.h  -  routines and wrappers around soundcard.h
		    for sound programming with Unix Sound System

	(c) 1998 Martin Pitt as Public Domain
	use, modify and spread freely!

	Version: 1.0
*/

#ifndef SOUND_H
#define SOUND_H

#include <stdio.h>  // fprintf()

typedef unsigned short WORD;
typedef unsigned long ULONG;

//* SOUND FORMAT HEADERS ********************************************

typedef struct 
{
	//* RIFF header ******************
    char rID[4];		// magic "RIFF"
    ULONG rLen; 		// length of RIFF chunk block

	//* WAVE chunk header ************
    char wID[4];		// magic "WAVE"

	//* Format chunk *****************
    char fID[4];		// magic "fmt "
    ULONG fLen;			// length of fmt-chunk
    WORD wFormatTag;		// 1 = PCM (8 bit unsigned uncompressed)
    WORD nChannels;		// 1 = mono, 2 = stereo
    ULONG nSamplesPerSec;	// playback frequency
    ULONG nAvgBytesPerSec;	// = nChannels*nSamplesPerSec*(nBitsPerSample/8)
    WORD nBlockAlign;		// = nChannels * (nBitsPerSample/8) 
    WORD FormatSpecific;

	//* Data chunk header ************
    char dID[4];		// magic "data"
    ULONG dLen;			// length of following sample data
} WAVHeader;  // the actual wave data follows directly after the  header

//* errors *****************************************************************

enum SoundError {seOK, seFILENOTFOUND, seFILEFORMAT, seINITSOUNDCARD,
		 seFORMATUNKNOWN, seFORMATNOTPLAYABLE, seUNEXPECTEDEOF};

extern SoundError snd_errno;

const char* snd_strerror ();
void snd_perror (const char*); // prints string + error on stderr

//* functions **************************************************************

int snd_initDevice (int access, unsigned freq, bool stereo, int format, WORD blockSizeExp = 0);
    // access: O_RDONLY, O_WRONLY or (if absolutely neccessary, but not recommended) O_RDWR
    // format: use the AFMT_* constants from soundcard.h
    // blockSizeExp: block buffer size = 2**blockSizeExp; 0: default (let driver decide)
    // returns file descriptor or -1 on error

int snd_playWAV (const char* fname);
    // -> -1: error, 0: OK

//* INLINE FUNCTIONS ***********************************************************

inline
void
snd_perror (const char* s)
{
    fprintf (stderr, "\n\n%s: %s\n\n", s,  snd_strerror());
}

#endif // !defined SOUND_H


